<?php

namespace Teknisa\CNC\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DB\StorageProcedure\Param;



use Teknisa\CNC\Services\Cnc02500SerieEstabelecimentoService as Service;
use Teknisa\CNC\Utilities;
use Teknisa\Libs\Util\Environment;

class Cnc02500SerieEstabelecimento extends \Teknisa\CNC\Controllers\TeknisaCrud {
    
    const SERIES = 'cnc02500_serie_estabelecimento';
    const SERIESCAD = 'cnc02500_serie_estabelecimento_cad';
    const ESTABELECIMENTOS= 'cnc02500_serie_estabelecimento_estabelecimentos';
    protected $service;
    protected $parameterBagColumns;
    protected $environment;
    protected $dataSourceName;
    protected $salesMessage;
    protected $utilCnc;

    public function __construct ($crudService, ParameterBag $parameterBagColumns, Service $service, 
                                 Environment $environment, Utilities\SalesMessageCNC $salesMessage,
                                 Utilities\UtilCNC $utilCnc
                                 ) {
	    parent::__construct($crudService, $parameterBagColumns);
        $this->service      = $service;
        $this->environment  = $environment;
        $this->salesMessage = $salesMessage;
        $this->utilCnc      = $utilCnc;
	}
	
	public function save(DTO\Request $request, DTO\Response $response) {
		try {
            $row = $request->getDataSet()->getRows()[0];
            $row["NRORG"] = $this->environment->getNrOrgTrab();
            $result = $this->service->save($row);
            if($result == true){
    		    $message = array($this->salesMessage->getMessage('criado'));
            } else{
                $message = array($this->salesMessage->getMessage('naoCriado'));
            }
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc02500_serie_estabelecimento_cad', $message));
    	} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function findSerie(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::SERIES;
        parent::find($request,$response);
    }
    
    public function findSerieCad(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::SERIESCAD;
        parent::find($request,$response);
    }
    
	public function findEstabelec(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = self::ESTABELECIMENTOS;
        parent::find($request,$response);
    }
    
    public function findEstabelecSelect(Request\Filter $request, Response $response){
        $allFieldsLabel = 'CDESTABELEC|NMESTABELEC|DSSERIE';
        try {
            $filterConditions      = $request->getFilterCriteria()->getConditions();
            $params['NRORG']       = $this->environment->getNrOrgTrab();
            $params['NRESTABELEC'] = $this->utilCnc->getParamValueFromFilter($filterConditions, "NMESTABELEC");

            $params['TNRESTABELEC'] = null;
            if (!is_array($params['NRESTABELEC']))
                $params['TNRESTABELEC'] = "T";
            else 
                $params['NRESTABELEC'] = $this->utilCnc->buildParamValue($params['NRESTABELEC']);
                
            $params['ALL_FIELDS'] = $this->utilCnc->getParamValueFromFilter($filterConditions, $allFieldsLabel);

            if(empty($params['ALL_FIELDS'])) {
                $params['ALL_FIELDS'] = '';
                $params['TALL_FIELDS'] = 'T';
            } else {
                $params['ALL_FIELDS'] = str_replace('%', '', $params['ALL_FIELDS'][0]['_ALL']);
                $params['TALL_FIELDS'] = 'F';
            }       
            
            $result = $this->service->findEstabelecSelect($params);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('cnc02500_serie_estabelecimento_select', $result));
        } catch (\Exception $e) {
            $response->addMessage(new Message($e->getMessage()));
        }
    }
    
    public function findSerieCadCheck(Request\Filter $request, Response $response) {
        try {
            $params = $request->getFilterCriteria()->getConditions();
            $params['NRORG'] = $this->environment->getNrOrgTrab();
            $erros = 0;
            date_default_timezone_set('America/Sao_Paulo');
            $resultado[] = "Log de Cadastro de Série por Estabelecimento";
            
            $cabecalhoErro = false;
            foreach($params[0]["value"] as $paramethers) {
                $params["NRESTABELEC"]["value"] = $paramethers;
                $params["NMESTABELEC"] = $this->service->findSerieCadCheck($params)[0]["NMESTABELEC"];
                $params["CDINSCESTA"]  = $this->service->findSerieCadCheck($params)[0]["CDINSCESTA"];
                $params["CDINSCMUNI"]  = $this->service->findSerieCadCheck($params)[0]["CDINSCMUNI"];
                $params["CDEMPRESA"]   = $this->service->findSerieCadCheck($params)[0]["CDEMPRESA"];
                $seriesEstabelec = $this->service->getEstabelecSerie($params);
                $isEqual = false;
                foreach($seriesEstabelec as $serieEst) {
                    if($serieEst["DSSERIE"] == $params[1]["value"]){
                       $isEqual = true;
                    }
                }
                if($isEqual == false){
                     if (!$cabecalhoErro) {
                        $resultado[] = str_pad('-', 100, "-", STR_PAD_LEFT);
                        $resultado[] = "Lista de estabelecimentos sem a série cadastrada na Unidade:";
                        $resultado[] = "";
                        $cabecalhoErro = true;
                    }
                    $resultado[] = "- ".$params["NMESTABELEC"];
                    $erros++;
                }
            }
            if($erros != 0){
                $resultado[] = "";
                $resultado[] = "Quantidade de Estabelecimentos sem o cadastro da série na Unidade relacionada: ".$erros;
                $resultado[] = "Os demais Estabelecimentos tiveram a série cadastrada com sucesso.";
            }
            $resultado[] = str_pad('_', 78, "_", STR_PAD_LEFT);
            $resultado[] = "Cadastramento automático executado com sucesso!";
            

            $response->addDataset(new DataSet('log', $resultado));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}