<?php

namespace Teknisa\CNC\Services;

use Zeedhi\Framework\DB\StoredProcedure\Param;

use Teknisa\CNC\Factories\Cnc01800ReconciliacaoFactory;

class Cnc01800ReconciliacaoService {
    
    protected $factory;
    
    public function __construct(Cnc01800ReconciliacaoFactory $factory) {
        $this->factory = $factory;
    }
    
    public function reconciliar ($row) {
        $logReconciliaAutomatico = [];
        $estabelecForaFinanceiro = [];
        $estabelecimentos = $row["LABELESTABELEC"];
        foreach ($estabelecimentos as $estabelecimento) {
            $row["NMESTABELEC"] = $estabelecimento;
            
                $row["P_SEQESTABELEC"] = $this->buildInputParam("P_SEQESTABELEC", $row["NMESTABELEC"]);
                $row["P_DTINICIAL"] = $this->buildInputParam("P_DTINICIAL", $row["DTVENDA"]["start"]);
                $row["P_DTFINAL"] = $this->buildInputParam("P_DTFINAL", $row["DTVENDA"]["end"]);
                $this->factory->reconciliar($row);
                
            if ($this->validateAccountingPeriod($row)) {
                $this->factory->financeiro($row);
            }else{
                $nmestabelec = $this->factory->verifyEstabelecName(array("NRESTABELEC" => $estabelecimento))["NMESTABELEC"];
                $estabelecForaFinanceiro[] = $nmestabelec;
            }
        }
        if(count($estabelecForaFinanceiro)>0){
            $logReconciliaAutomatico[] = "Log de Reconciliação Automática";
            $logReconciliaAutomatico[] = "-----------------------------------------------------------------------------------------------------------------------------------------------";
            $logReconciliaAutomatico[] = "A reconciliação automática foi efetuada para todos os estabelecimentos selecionados.";
            $logReconciliaAutomatico[] = "-----------------------------------------------------------------------------------------------------------------------------------------------";
            $logReconciliaAutomatico[] = "Os Estabelecimentos abaixo estão fora do período contábil para a data parametrizada e não foram incluidos no processo de refatoração do financeiro: ";
            $logReconciliaAutomatico = array_merge($logReconciliaAutomatico, $estabelecForaFinanceiro);
        }
        
        return $logReconciliaAutomatico;
    }
    
    public function validateAccountingPeriod ($row) {
        $result = $this->factory->validateAccountingPeriod(array(
            ":NRESTABELEC"  => $row["NMESTABELEC"],
            ":DT_INI"       => $row["DTVENDA"]["start"],
            ":DT_FIN"       => $row["DTVENDA"]["end"]
        ));
        
        return isset($result[0]);
    }
    
    private function buildInputParam ($paramName, $paramValue) {
        return new Param($paramName, Param::PARAM_INPUT, $paramValue, Param::PARAM_TYPE_STR, strlen($paramValue));
    }
    
    public function verifyDivergences ($params) {
        foreach($params["LABELESTABELEC"] as $estabelecimento){
            $result = $this->factory->verifyDivergences(array(
                ":NRESTABELEC"  => $estabelecimento));
            if ($result["COUNT"] > 0){
                throw new \Exception("DIVERGENCE_FOUND", 1);
            }   
        }
        
    }
}